#include "ola_receiver.h"
#include <ola/DmxBuffer.h>
#include <ola/Logging.h>
#include <ola/client/ClientWrapper.h>
#include "spisender.h"

// Important to go from one.
// Else modify the NewDmx callback
static unsigned int universes[6] = {1, 2, 3, 4, 5, 6};
// The 'n'th universe (n: 1-6) goes from (rows[n-1]) to (rows[n]-1) inclusive
static int universe_rows[7] = {0, 5, 10, 15, 20, 25, 26};

static void RegisterComplete(const ola::client::Result &result);
static void NewDmx(const ola::client::DMXMetadata &metadata,
            const ola::DmxBuffer &data);

static SpiSender* spisender = NULL;

static ola::client::OlaClientWrapper* wrapper = NULL;


void ola_init(SpiSender* sender)
{
    if (wrapper != NULL) return;
    spisender = sender;
    ola::InitLogging(ola::OLA_LOG_INFO, ola::OLA_LOG_STDERR);
    wrapper = new ola::client::OlaClientWrapper();
    if (!wrapper->Setup())
        exit(1);
    ola::client::OlaClient *client = wrapper->GetClient();
    // Set the callback and register our interest in this universe
    client->SetDMXCallback(ola::NewCallback(&NewDmx));
    for (size_t i = 0; i < 6; i++)
    {
        client->RegisterUniverse(universes[i], ola::client::REGISTER,
            ola::NewSingleCallback(&RegisterComplete));
    }
    
    //wrapper.GetSelectServer()->Run();
}

void ola_run_once()
{
    wrapper->GetSelectServer()->RunOnce();
}

void ola_deinit()
{
    delete wrapper;
    wrapper = NULL;
}

// Called when universe registration completes.
static void RegisterComplete(const ola::client::Result &result)
{
    if (!result.Success())
    {
        OLA_WARN << "Failed to register universe: " << result.Error();
    }
    printf("RegisterComplete\n");
}
// Called when new DMX data arrives.
static void NewDmx(const ola::client::DMXMetadata &metadata,
            const ola::DmxBuffer &data)
{
    MxFrame frame;
    int startY = universe_rows[metadata.universe-1];
    int endY = universe_rows[metadata.universe]-1;
    const uint8_t* arr = data.GetRaw();
    int k = 0;
    for (size_t y = startY; y <= endY; y++)
    {
        for (size_t x = 0; x < 32; x++)
        {
            frame.pixels[y][x].r = arr[k];
            frame.pixels[y][x].g = arr[k+1];
            frame.pixels[y][x].b = arr[k+2];
            k += 3;
        }
    }
    spisender->updateFrame(frame, startY, endY);
}
