#include "spisender.h"
#include "ola_receiver.h"
#include "wiringPi.h"
#include "wiringPiSPI.h"
#include <unistd.h>
#include <stdint.h>

pthread_mutex_t SpiSender::mutex_frame = PTHREAD_MUTEX_INITIALIZER;

SpiSender::SpiSender()
{
    wiringPiSetup();
    pinMode(readyPin, INPUT);
    pinMode(resetPin, OUTPUT);
    pullUpDnControl(readyPin, PUD_DOWN);
    digitalWrite(resetPin, LOW);
    usleep(1000);
    digitalWrite(resetPin, HIGH);
    usleep(1000); // Wait for startup (doesn't work without it)
    fd = wiringPiSPISetup(SPI_CHANNEL, SPI_SPEED);
    uint8_t dummy = 0;
    wiringPiSPIDataRW(SPI_CHANNEL, &dummy, 1);

    exit_flag = false;
    pthread_attr_t attr;
    pthread_attr_init(&attr);
    pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_JOINABLE);
    pthread_create(&sender_thread, &attr, SpiSender::sender_caller, (void*)this);
    pthread_attr_destroy(&attr);
}

SpiSender::~SpiSender()
{
    close(fd);
    exit_flag = true;
    pthread_join(sender_thread, NULL);
}

bool SpiSender::sendFrame(const SpiFrame &frame) const
{
    if (!canSend()) return false;
    uint8_t dummy = 0;
    wiringPiSPIDataRW(SPI_CHANNEL, (uint8_t*)frame.data, 64*24*2);
    wiringPiSPIDataRW(SPI_CHANNEL, &dummy, 1);
    return true;
}

bool SpiSender::canSend() const
{
    return digitalRead(readyPin);
}
void* SpiSender::sender_caller(void* arg)
{
    SpiSender* sender = (SpiSender*)arg;
    printf("Spi thread started.\n");
    sender->sender_loop();
    pthread_exit(NULL);
}

void SpiSender::sender_loop()
{
    while (!exit_flag)
    {
        if (canSend())
        {
            pthread_mutex_lock(&mutex_frame);
            sendFrame(SpiFrame(frame));
            pthread_mutex_unlock(&mutex_frame);
        }
        //printf("Frame sent.\n");
        usleep(6000);
    }
    printf("SpiSender thread exit");
}

void SpiSender::exit()
{
    exit_flag = true;
}

void SpiSender::updateFrame(const MxFrame& newFrame, int startY, int endY)
{
    pthread_mutex_lock(&mutex_frame);
    for (size_t y = startY; y <= endY; y++)
    {
        for (size_t x = 0; x < 32; x++)
        {
            frame.pixels[y][x] = newFrame.pixels[y][x];
        }
    }
    pthread_mutex_unlock(&mutex_frame);
}
